@@ -14,7 +14,7 @@ our $Debug;
 
 *Debug = \$Rose::DB::Object::Metadata::Debug;
 
-our $VERSION = '0.50';
+our $VERSION = '0.58';
 
 use Rose::Class::MakeMethods::Generic
 (
@@ -58,7 +58,7 @@ sub auto_generate_columns
 {
   my($self) = shift;
 
-  my($db, $class, %columns, $schema, $table);
+  my($db, $class, %columns, $catalog, $schema, $table);
 
   eval
   {
@@ -73,15 +73,19 @@ sub auto_generate_columns
 
     my $table_unquoted = $db->unquote_table_name($table);
 
-    $schema = $db->schema;
-    $schema = $db->default_implicit_schema  unless(defined $schema);
+    $catalog = $self->select_catalog($db);
+    $schema  = $self->select_schema($db); 
+    $schema  = $db->default_implicit_schema  unless(defined $schema);
+    
+    $schema  = lc $schema   if(defined $schema && $db->likes_lowercase_schema_names);
+    $catalog = lc $catalog  if(defined $catalog && $db->likes_lowercase_catalog_names);
 
-    my $sth = $dbh->column_info($db->catalog, $schema, $table_unquoted, '%');
+    my $sth = $dbh->column_info($catalog, $schema, $table_unquoted, '%');
 
     unless(defined $sth)
     {
       no warnings; # undef strings okay
-      die "No column information found for catalog '", $db->catalog,
+      die "No column information found for catalog '", $catalog,
           "' schema '", $schema, "' table '", $table_unquoted, "'";
     }
 
@@ -93,7 +97,7 @@ sub auto_generate_columns
 
         $col_info->{'TABLE_NAME'} = $db->unquote_table_name($col_info->{'TABLE_NAME'});
 
-        next COLUMN unless($col_info->{'TABLE_CAT'}   eq $db->catalog &&
+        next COLUMN unless($col_info->{'TABLE_CAT'}   eq $catalog &&
                            $col_info->{'TABLE_SCHEM'} eq $schema &&
                            $col_info->{'TABLE_NAME'}  eq $table_unquoted);
       }
@@ -114,7 +118,7 @@ sub auto_generate_columns
   {
     no warnings; # undef strings okay
     Carp::croak "Could not auto-generate columns for class $class - ",
-                ($@ || "no column info found for catalog '" . $db->catalog .
+                ($@ || "no column info found for catalog '" . $catalog .
                 "' schema '" . $schema . "' table '$table'");
   }
 
@@ -285,7 +289,7 @@ sub auto_retrieve_primary_key_column_names
     Carp::croak "Useless call to auto_retrieve_primary_key_column_names() in void context";
   }
 
-  my($db, $class, @columns, $schema);
+  my($db, $class, @columns, $catalog, $schema);
 
   eval
   {
@@ -298,15 +302,19 @@ sub auto_retrieve_primary_key_column_names
 
     my $table_unquoted = $db->unquote_table_name($table);
 
-    $schema = $db->schema;
+    $catalog = $self->select_catalog($db);
+    $schema = $self->select_schema($db);
     $schema = $db->default_implicit_schema  unless(defined $schema);
 
-    my $sth = $dbh->primary_key_info($db->catalog, $schema, $table_unquoted);
+    $schema  = lc $schema   if(defined $schema && $db->likes_lowercase_schema_names);
+    $catalog = lc $catalog  if(defined $catalog && $db->likes_lowercase_catalog_names);
+
+    my $sth = $dbh->primary_key_info($catalog, $schema, $table_unquoted);
 
     unless(defined $sth)
     {
       no warnings; # undef strings okay
-      die "No primary key information found for catalog '", $db->catalog,
+      die "No primary key information found for catalog '", $catalog,
           "' schema '", $schema, "' table '", $table, "'";
     }
 
@@ -318,7 +326,7 @@ sub auto_retrieve_primary_key_column_names
 
         $pk_info->{'TABLE_NAME'} = $db->unquote_table_name($pk_info->{'TABLE_NAME'});
 
-        next PK  unless($pk_info->{'TABLE_CAT'}   eq $db->catalog &&
+        next PK  unless($pk_info->{'TABLE_CAT'}   eq $catalog &&
                         $pk_info->{'TABLE_SCHEM'} eq $schema &&
                         $pk_info->{'TABLE_NAME'}  eq $table_unquoted);
       }
@@ -336,7 +344,7 @@ sub auto_retrieve_primary_key_column_names
   {
     $@ = 'no primary key coumns found'  unless(defined $@);
     Carp::croak "Could not auto-retrieve primary key columns for class $class - ",
-                ($@ || "no primary key info found for catalog '" . $db->catalog .
+                ($@ || "no primary key info found for catalog '" . $catalog .
                 "' schema '" . $schema . "' table '" . lc $self->table, "'");
   }
 
@@ -365,25 +373,29 @@ sub auto_generate_foreign_keys
     my $db  = $self->db;
     my $dbh = $db->dbh or die $db->error;
 
+    my $catalog = $self->select_catalog($db);
+    my $schema  = $self->select_schema($db); 
+    $schema = $db->default_implicit_schema  unless(defined $schema);
+
+    $schema  = lc $schema   if(defined $schema && $db->likes_lowercase_schema_names);
+    $catalog = lc $catalog  if(defined $catalog && $db->likes_lowercase_catalog_names);
+
     my $table = $db->likes_lowercase_table_names ? lc $self->table : $self->table;
 
     my $sth = $dbh->foreign_key_info(undef, undef, undef,
-                                     $db->catalog, $db->schema, $table);
+                                     $catalog, $schema, $table);
 
     # This happens when the table has no foreign keys
     return  unless(defined $sth);
 
     my(%fk, @fk_info);
 
-    my $schema = $db->schema;
-    $schema = $db->default_implicit_schema  unless(defined $schema);
-
     FK: while(my $fk_info = $sth->fetchrow_hashref)
     {
       CHECK_TABLE: # Make sure this column is from the right table
       {
         no warnings; # Allow undef coercion to empty string
-        next FK  unless($fk_info->{'FK_TABLE_CAT'}   eq $db->catalog &&
+        next FK  unless($fk_info->{'FK_TABLE_CAT'}   eq $catalog &&
                         $fk_info->{'FK_TABLE_SCHEM'} eq $schema &&
                         $fk_info->{'FK_TABLE_NAME'}  eq $table);
       }
@@ -917,6 +929,16 @@ sub auto_init_primary_key_columns
     Carp::croak "Could not retrieve primary key columns for class ", ref($self);
   }
 
+  # Wipe pk defaults because stupid MySQL adds them implicitly
+  if($self->db->driver eq 'mysql')
+  {
+    foreach my $name (@$primary_key_columns)
+    {
+      my $column = $self->column($name) or next;
+      $column->default(undef);
+    }
+  }
+
   $self->primary_key_columns(@$primary_key_columns);
 
   return;
@@ -1064,7 +1086,7 @@ sub auto_init_one_to_many_relationships
     # XXX: skip of there's already a relationship with the same id
 
     # Add the one to many relationship to the foreign class
-    my $name = $cm->auto_table_to_relationship_name_plural($self->table);
+    my $name = $cm->auto_relationship_name_one_to_many($self->table, $class);
 
     unless($f_meta->relationship($name))
     {
